import fs from 'fs/promises';
import type { Stats } from 'fs';
import { Hashes } from '../hash';
import { basename, extname } from 'path';

interface SequenceObject {
	path : string,
	hash : string,
	name : string
}

interface VideoObject {
	path : string,
	hash : string,
	name : string
}

const fileExtensions : string[] = [
	'.mp4',
	'.mkv',
	'.mov'
];

export class Files {
	public static async exists (path : string) : Promise<boolean> {  
		try {
			await fs.access(path);
			return true;
		} catch {
			return false;
		}
	}

	public static async enumerateSequences (path : string) : Promise<SequenceObject[]> {
		const dirs : SequenceObject[] = [];
		let all : string[];
		let stats : Stats;
		try {
			all = await fs.readdir(path)
		} catch (err) {
			throw err;
		}

		for (let elem of all) {
			try {
				stats = await fs.lstat(elem);
    			if (stats.isDirectory()) {
    				dirs.push({
    					path : elem,
    					hash : Hashes.stringHash(elem),
    					name : basename(elem)
    				});
    			}
			} catch (err) {
				//
			}
		}

		return dirs;
	}

	public static async enumerateVideos (path : string) : Promise<VideoObject[]> {
		const videos : VideoObject[] = [];
		let all : string[];
		let stats : Stats;

		try {
			all = await fs.readdir(path)
		} catch (err) {
			throw err;
		}

		for (let elem of all) {
			try {
				stats = await fs.lstat(elem);
    			if (stats.isFile() && fileExtensions.indexOf(extname(basename(elem))) !== -1) {
    				videos.push({
    					path : elem,
    					hash : Hashes.stringHash(elem),
    					name : basename(elem)
    				});
    			}
			} catch (err) {
				//
			}
		}

		return videos;
	}
}

export type { SequenceObject, VideoObject };
module.exports = { Files };
